@@ -10,7 +10,27 @@ $substvar{'Tab'}= "\t";
 $maxsubsts=50;
 
 $progname= $0; $progname= $& if $progname =~ m,[^/]+$,;
-@fowner = (getpwnam(getlogin()))[2,3];
+
+if (defined ($ENV{'LOGNAME'})) {
+    if (!defined (getlogin ())) { 
+	warn (sprintf ('no utmp entry available, using value of LOGNAME ("%s")', $ENV{'LOGNAME'})); 
+    } else {
+	if (getlogin () ne $ENV{'LOGNAME'}) { 
+	    warn (sprintf ('utmp entry ("%s") does not match value of LOGNAME ("%s"); using "%s"',
+			   getlogin(), $ENV{'LOGNAME'}, $ENV{'LOGNAME'}));
+	}
+    }
+    @fowner = getpwnam ($ENV{'LOGNAME'});
+    if (! @fowner) { die (sprintf ('unable to get login information for username "%s"', $ENV{'LOGNAME'})); }
+} elsif (defined (getlogin ())) {
+    @fowner = getpwnam (getlogin ());
+    if (! @fowner) { die (sprintf ('unable to get login information for username "%s"', getlogin ())); }
+} else {
+    warn (sprintf ('no utmp entry available and LOGNAME not defined; using uid of process (%d)', $<));
+    @fowner = getpwuid ($<);
+    if (! @fowner) { die (sprintf ('unable to get login information for uid %d', $<)); }
+}
+@fowner = @fowner[2,3];
 
 sub capit {
     return defined($capit{lc $_[0]}) ? $capit{lc $_[0]} :
@@ -26,7 +46,7 @@ sub findarch {
 
 sub substvars {
     my ($v) = @_;
-    my $lhs,$vn,$rhs,$count;
+    my ($lhs,$vn,$rhs,$count);
     $count=0;
     while ($v =~ m/\$\{([-:0-9a-z]+)\}/i) {
         $count < $maxsubsts ||
@@ -44,6 +64,7 @@ sub substvars {
 }
 
 sub outputclose {
+    my ($dosubstvars) = @_;
     for $f (keys %f) { $substvar{"F:$f"}= $f{$f}; }
     if (length($varlistfile)) {
         $varlistfile="./$varlistfile" if $varlistfile =~ m/\s/;
@@ -62,7 +83,9 @@ sub outputclose {
     }
     for $f (sort { $fieldimps{$b} <=> $fieldimps{$a} } keys %f) {
         $v= $f{$f};
-        $v= &substvars($v);
+        if ($dosubstvars) {
+	    $v= &substvars($v);
+	}
         $v =~ m/\S/ || next; # delete whitespace-only fields
         $v =~ m/\n\S/ && &internerr("field $f has newline then non whitespace >$v<");
         $v =~ m/\n[ \t]*\n/ && &internerr("field $f has blank lines >$v<");
