@@ -57,7 +57,7 @@ do
 	-sd)	sourcestyle=-sd ;;
 	-tc)	cleansource=true ;;
 	-b)	binaryonly=-b ;;
-	-B)	binaryonly=-b; binarytarget=binary-arch ;;
+	-B)	binaryonly=-B; binarytarget=binary-arch ;;
 	-v*)	since="$value" ;;
 	-m*)	maint="$value" ;;
 	-C*)	descfile="$value" ;;
@@ -81,6 +81,8 @@ curd="`pwd`"
 dirn="`basename \"$curd\"`"
 mustsetvar package "`dpkg-parsechangelog | sed -n 's/^Source: //p'`" "source package"
 mustsetvar version "`dpkg-parsechangelog | sed -n 's/^Version: //p'`" "source version"
+if [ -n "$maint" ]; then maintainer="$maint"; 
+else mustsetvar maintainer "`dpkg-parsechangelog | sed -n 's/^Maintainer: //p'`" "source version"; fi
 test "${opt_a}" \
 	|| mustsetvar arch "`dpkg --print-architecture`" "build architecture"
 
@@ -89,7 +91,7 @@ pv="${package}_${sversion}"
 pva="${package}_${sversion}${arch:+_${arch}}"
 
 signfile () {
-	$pgpcommand +clearsig=on -fast <"../$1" >"../$1.asc"
+	$pgpcommand -u "$maintainer" +clearsig=on -fast <"../$1" >"../$1.asc"
 	echo
 	mv -- "../$1.asc" "../$1"
 }
@@ -110,7 +112,9 @@ if [ x$binaryonly = x ]; then
 fi
 withecho debian/rules build
 withecho $rootcommand debian/rules $binarytarget
-$signsource "$pv.dsc"
+if [ x$binaryonly = x ]; then
+        $signsource "$pv.dsc"
+fi
 chg=../"$pva.changes"
 withecho dpkg-genchanges "$@" >"$chg"
 
