@@ -87,8 +87,10 @@ while (@ARGV) {
     }
 }
 
-$arch=`dpkg --print-architecture`;
-$? && &subprocerr("dpkg --print-architecture");
+$arch = $override{Architecture} or do {
+       $arch=`dpkg --print-architecture`;
+       $? && &subprocerr("dpkg --print-architecture");
+};
 $arch =~ s/\n$//;
 
 &parsechangelog;
@@ -206,7 +208,8 @@ chown(@fowner, "$fileslistfile.new")
 if (open(X,"< $fileslistfile")) {
     while (<X>) {
         s/\n$//;
-        next if m/^([-+0-9a-z.]+)_[^_]+_\w+\.deb / && $1 eq $oppackage;
+        next if m/^([-+0-9a-z.]+)_[^_]+_(\w+)\.deb /
+                && ($1 eq $oppackage) && ($2 eq $arch);
         print(Y "$_\n") || &syserr("copy old entry to new files list file");
     }
 } elsif ($! != ENOENT) {
@@ -230,7 +233,7 @@ if (!$stdout) {
     open(STDOUT,"> $cf.new") ||
         &syserr("cannot open new output control file \`$cf.new'");
 }
-&outputclose;
+&outputclose(1);
 if (!$stdout) {
     rename("$cf.new","$cf") || &syserr("cannot install output control file \`$cf'");
 }
